﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using C1.Win.C1FlexGrid;

namespace CustomControls
{
    /// <summary>
    /// FlexAccordion is an accordion-like control implemented from C1FlexGrid.
    /// 
    /// It was modeled specifically to fit this Control Explorer, but can be modified for other uses.
    /// This model uses the IsNode property to distinguish between headers and items.
    /// </summary>
    public partial class FlexAccordion : C1FlexGrid
    {
        # region private members

        private int _expandedIndex;
        private int _selectedItemIndex;
        private int _animationDelay;
        private bool _canExpand;
        private CellStyle _headerStyle;
        private CellStyle _itemStyle;

        # endregion

        # region constructor

        public FlexAccordion()
        {
            InitializeComponent();
            _headerStyle = this.Styles.Add("Header");
            _itemStyle = this.Styles.Add("Item");
            _expandedIndex = -1;
            _selectedItemIndex = -1;
            _canExpand = true;
            this.ScrollBars = ScrollBars.None;
            this.SelectionMode = SelectionModeEnum.Row;
        }
        # endregion

        # region public members
        /// <summary>
        /// Gets or sets the currently expanded row index
        /// </summary>
        public int ExpandedIndex
        {
            get { return _expandedIndex; }
            set { _expandedIndex = value; }
        }

        /// <summary>
        /// Gets or sets the currently selected item
        /// </summary>
        public int SelectedItemIndex
        {
            get { return _selectedItemIndex; }
            set
            {
                if (_selectedItemIndex != -1)
                {
                    this.SetCellStyle(_selectedItemIndex, 0, _itemStyle);
                }
                _selectedItemIndex = value;
            }
        }

        /// <summary>
        /// Gets or sets the animation delay in milliseconds.
        /// </summary>
        public int AnimationDelay
        {
            get { return _animationDelay; }
            set { _animationDelay = value; }
        }

        /// <summary>
        /// Gets or sets whether the control can expand/collapse
        /// </summary>
        public bool CanExpand
        {
            get { return _canExpand; }
            set { _canExpand = value; }
        }

        /// <summary>
        /// Gets or sets the CellStyle object associated with item headers.
        /// </summary>
        public CellStyle HeaderStyle
        {
            get { return _headerStyle; }
            set { _headerStyle = value; }
        }

        /// <summary>
        /// Gets or sets the CellStyle object associated with items.
        /// </summary>
        public CellStyle ItemStyle
        {
            get { return _itemStyle; }
            set { _itemStyle = value; }
        }

        # endregion

        # region implementation

        protected override void OnMouseClick(MouseEventArgs e)
        {
            base.OnMouseClick(e);
            if (this.MouseRow != -1)
            {
                Row hitRow = this.Rows[this.MouseRow];
                if (hitRow.Index == _expandedIndex)
                {
                    //Do nothing
                }
                else if (hitRow.IsNode && this._canExpand)
                {
                    //Expand/Collapse items
                    Expand(hitRow.Index);
                    _expandedIndex = hitRow.Index;
                }
                else if (!hitRow.IsNode)
                {
                    this.SelectedItemIndex = hitRow.Index;
                }
            }
            this._canExpand = true;  
        }

        //This method collapses the currently expanded items (if any) and expands at the passed row index.
        protected void Expand(int expandRow)
        {
            int collapseRow = _expandedIndex + 1;
            expandRow += 1;
            bool done = false;
            bool collapsing = false;
            bool expanding = false;
            while (!done)
            {
                //Update the current collapsing row index
                if (collapseRow < this.Rows.Count)
                    collapsing = !this.Rows[collapseRow].IsNode;
                else collapsing = false;

                //Update the current expanding row index
                if (expandRow < this.Rows.Count)
                    expanding = !this.Rows[expandRow].IsNode;
                else expanding = false;

                if (expanding || collapsing)
                {
                   //Collapse and expand 1 row each at a time
                    if (collapsing)
                        this.Rows[collapseRow].Visible = !this.Rows[collapseRow].Visible;
                    if (expanding)
                        this.Rows[expandRow].Visible = !this.Rows[expandRow].Visible;
                    
                    //Redraw FlexGrid to reflect new visible rows and sleep for animation effect
                    this.Refresh();
                    System.Threading.Thread.Sleep(_animationDelay);
                }
                else
                {
                    //No more rows to expand or collapse
                    done = true;
                }
              
                //if we could expand/collapse continue to next row
                if (collapsing) collapseRow += 1;
                if (expanding) expandRow += 1;                 
            }
            updateScrollButtons();
        }

        //Handle mouseover styles
        protected override void OnMouseEnterCell(RowColEventArgs e)
        {
            base.OnMouseEnterCell(e);
            if (!this.Rows[e.Row].IsNode)
            {
                this.SetCellStyle(e.Row, 0, this.Styles["Highlight"]);
            }
        }
        protected override void OnMouseLeaveCell(RowColEventArgs e)
        {
            base.OnMouseLeaveCell(e);
            if (!this.Rows[e.Row].IsNode && e.Row != _selectedItemIndex)
            {
                this.SetCellStyle(e.Row, 0, _itemStyle);
            }
        }

        //Scroll functions
        private void updateScrollButtons()
        {
            if (this.BottomRow < LastVisibleRow())
            {
                btnBottomScroll.Visible = true;
                btnBottomScroll.Location = new Point(this.Width - btnBottomScroll.Width, this.Height - btnBottomScroll.Height);
            }
            else
            {
                btnBottomScroll.Visible = false;
            }

            if (this.TopRow > 0)
            {
                btnTopScroll.Visible = true;
                btnTopScroll.Location = new Point(this.Width - btnTopScroll.Width, 0);
            }
            else
            {
                btnTopScroll.Visible = false;
            }
        }

        //This function determines the last visible row of the grid, which may be different than the last row.
        private int LastVisibleRow()
        {
            int i = this.Rows.Count - 1;
            while (!this.Rows[i].Visible)
            {
                i -= 1;
            }
            return i;
        }     

        protected override void OnSizeChanged(EventArgs e)
        {
            base.OnSizeChanged(e);
            if(btnBottomScroll != null && this.Rows.Count > 0)
                updateScrollButtons();
        }

        protected override void OnAfterScroll(RangeEventArgs e)
        {
            base.OnAfterScroll(e);
            updateScrollButtons();
        }

        private void btnTopScroll_Click(object sender, EventArgs e)
        {
            //Scroll up
            scroll(-1);
        }

        private void btnBottomScroll_Click(object sender, EventArgs e)
        {
            //Scroll down
            scroll(2);
        }

        //This function does a custom scroll by only counting visible rows
        private void scroll(int dir)
        {
            int i = this.TopRow + dir;
            while (!this.Rows[i].Visible)
            {
                i += dir;
            }
            this.TopRow = i;
        }

        # endregion

    }
}
